@@ -17,9 +17,15 @@ h.listDataTags()
 
 from __future__ import (absolute_import, division, print_function, unicode_literals)
 import pkg_resources
+from eigeningenuity.core import EigenServer
 from eigeningenuity.historian import get_historian, list_historians, get_default_historian_name
+from eigeningenuity.elastic import get_elastic
+from eigeningenuity.events import get_eventlog
+from eigeningenuity.assetmodel import get_assetmodel
+from eigeningenuity.smartdash import get_smartdash
+from eigeningenuity.sql import get_sql
 
 from eigeningenuity.core import *
 
-__all__ = ["get_historian", "get_assetmodel", "get_elastic", "getsql", "get_eventlog", "list_historians", "get_default_historian_name", "EigenServer"]
+__all__ = ["get_historian", "get_assetmodel", "get_elastic", "get_sql", "get_eventlog", "get_smartdash", "list_historians", "get_default_historian_name", "EigenServer"]
 __version__ = pkg_resources.require("python-eigen-ingenuity")[0].version
