@@ -46,7 +46,7 @@ class EigenServer (object):
                 else:
                     baseurl = name + "/"
             else:
-                baseurl = "http://" + name + "/"
+                baseurl = "https://" + name + "/"
         except:
             baseurl = "http://localhost:8080/"
 
