@@ -41,15 +41,65 @@ class ElasticConnection (object):
         url = self.baseurl + "?cmd=" + urlquote(cmd)
         return _do_eigen_json_request(url,**params)
 
-    def _testConnection(self,instance):
+    def _testConnection(self,instance,index):
         """Preflight Request to verify connection to ingenuity"""
         try:
-            status = requests.get(self.baseurl + "?cmd=DODIRECTSEARCH&clientname=" + instance).status_code
+            status = requests.get(self.baseurl + "?cmd=DODIRECTSEARCH&clientname=" + instance + "&index=" + index + "&search=%7B%7D").status_code
             if status != 200:
                 raise ConnectionError(
                     "Invalid API Response from " + self.baseurl + "?cmd=DODIRECTSEARCH&clientname" + instance + ". Please check the base url is correct, the instance is running and has an elasticsearch database.")
         except (URLError, ConnectionError):
             raise ConnectionError ("Failed to connect to ingenuity instance at " + self.baseurl + "?cmd=DODIRECTSEARCH" + ". Please check the url is correct and the instance is up.")
+        
+        return status
+    
+    def listIndices(self,instance:str):
+        """Lists all indices in Elastic Database.
+
+        Args:
+            instance: The elasticsearch instance to query indices from.
+            
+        Returns:
+            List of all indices
+        """
+
+        args = {"clientname": instance}
+
+        mappings = self._doJsonRequest("GETMAPPINGS",args)
+
+        return mappings.keys()
+
+        
+    def checkIndices(self,instance:str,indices:str):
+        """Checks for indices in Elastic Database.
+
+        Args:
+            instance: The elasticsearch instance to query instances from.
+            indices: The elasticsearch indices to search for
+            
+        Returns:
+            Boolean indicating existence of the index, or dict of form {index: boolean}
+        """
+
+        foundIndices = self.listIndices(instance)
+        indices = force_list(indices)
+
+        result = {}
+
+        if len(indices) != 1:
+            for index in indices:
+                if index in foundIndices:
+                    result[index] = True
+                else:
+                    result[index] = False
+        else:
+            if indices[0] in foundIndices:
+                return True
+            else:
+                return False
+
+        return result
+
 
     def executeRawQuery(self,index:str, query:str, instance:str = "elasticsearch-int", output:str = "json", filepath:str = None):
         """Executes a raw cypher query against Elastic.
@@ -64,7 +114,7 @@ class ElasticConnection (object):
         Returns:
             Elasticsearch API response to the query, the format is dependent on the output parameter
         """
-        self._testConnection()
+        self._testConnection(instance,index)
         validOutputTypes = ["raw", "json", "df", "file"]
         if output not in validOutputTypes:
             raise ValueError("output must be one of %r." % validOutputTypes)
@@ -90,7 +140,6 @@ class ElasticConnection (object):
             with open(filepath + ".json", "w") as f:
                 f.write(json.dumps(response, indent=4))
 
-
 def get_elastic(eigenserver:EigenServer = None):
     """Instantiate an elasticsearch connection for the given EigenServer.
 
