@@ -36,10 +36,12 @@ class EventLog (object):
         success = []
         for chunk in event_chunks:
             data = {"events": chunk}
-            success.append(requests.post(url, json=data, verify=False))
-        return all(success)
+            resp = requests.post(url, json=data, verify=False)
+            success.append(resp)
 
-    def pushTo365Events(self, events:Union[dict,list,str]) -> bool:
+        return success
+
+    def pushTo365(self, events:Union[dict,list,str]) -> bool:
         """
         Push one or more events to the office 365 connector, only accepts pre-defined event structures
 
