@@ -18,7 +18,6 @@ eigeningenuity.core) or a full URL to a JSON Bridge Historian instance.
 """
 
 import json
-import datetime
 import math
 import logging
 import csv
@@ -99,24 +98,30 @@ class JsonBridgeHistorian (object):
             RemoteServerException: When the provided historian could not be found, or the API response contains an error
             RunTimeError/HTTPError: No response from the API
         """
-        validOutputTypes = ["raw", "json", "df", "csv"]
+        validOutputTypes = ["raw", "json", "df", "csv", "string"]
         if output not in validOutputTypes:
             raise ValueError("output must be one of %r." % validOutputTypes)
-        ret = {}
+        response = {}
 
         for tag in force_list(tags):
             args = {}
             args['tag'] = tag
-            ret[tag] = (self._doJsonRequest("getmeta", args))
+            response[tag] = (self._doJsonRequest("getmeta", args))
+
 
         if output == "raw":
-            return ret
-        if output == "json":
-            return json.dumps(ret)
-        if output == "df":
-            return jsonToDf(ret, True)
+            return response
+        elif output == "json":
+            if len(force_list(tags)) == 1:
+                return response[tags]
+            else:
+                return response
+        elif output == "string":
+            return json.dumps(response)
+        elif output == "df":
+            return jsonToDf(response, True)
         elif output == "csv":
-            return csvWriter(ret,self.historian,filepath,multi_csv,"GetMetaData")
+            return csvWriter(response,self.historian,filepath,multi_csv,"GetMetaData")
         return None
 
 
@@ -137,7 +142,7 @@ class JsonBridgeHistorian (object):
             RemoteServerException: When the provided historian could not be found, or the API response contains an error
             RunTimeError/HTTPError: No response from the API
         """
-        validOutputTypes = ["raw", "json", "df", "csv"]
+        validOutputTypes = ["raw", "json", "df", "csv", "string"]
         if output not in validOutputTypes:
             raise ValueError("output must be one of %r." % validOutputTypes)
         args = {}
@@ -153,6 +158,11 @@ class JsonBridgeHistorian (object):
         if output == "raw":
             return response
         elif output == "json":
+            if len(force_list(tags)) == 1:
+                return response[tags]
+            else:
+                return response
+        elif output == "string":
             return json.dumps(items)
         elif output == "df":
             return jsonToDf(items)
@@ -180,7 +190,7 @@ class JsonBridgeHistorian (object):
             RemoteServerException: When the provided historian could not be found, or the API response contains an error
             RunTimeError/HTTPError: No response from the API
         """
-        validOutputTypes = ["raw", "json", "df", "csv"]
+        validOutputTypes = ["raw", "json", "df", "csv", "string"]
         if output not in validOutputTypes:
             raise ValueError("output must be one of %r." % validOutputTypes)
         args = {}
@@ -197,6 +207,11 @@ class JsonBridgeHistorian (object):
         if output == "raw":
             return response
         elif output == "json":
+            if len(force_list(tags)) == 1:
+                return items[tags]
+            else:
+                return items
+        elif output == "string":
             return json.dumps(items)
         elif output == "df":
             return jsonToDf(items)
@@ -225,7 +240,7 @@ class JsonBridgeHistorian (object):
             RemoteServerException: When the provided historian could not be found, or the API response contains an error
             RunTimeError/HTTPError: No response from the API
         """
-        validOutputTypes = ["raw", "json", "df", "csv"]
+        validOutputTypes = ["raw", "json", "df", "csv", "string"]
         if output not in validOutputTypes:
             raise ValueError("output must be one of %r." % validOutputTypes)
         args = {}
@@ -244,6 +259,11 @@ class JsonBridgeHistorian (object):
         if output == "raw":
             return response
         elif output == "json":
+            if len(force_list(tags)) == 1:
+                return items[tags]
+            else:
+                return items
+        elif output == "string":
             return json.dumps(items)
         elif output == "df":
             return jsonToDf(items)
@@ -273,7 +293,7 @@ class JsonBridgeHistorian (object):
             RemoteServerException: When the provided historian could not be found, or the API response contains an error
             RunTimeError/HTTPError: No response from the API
         """
-        validOutputTypes = ["raw", "json", "df", "csv"]
+        validOutputTypes = ["raw", "json", "df", "csv", "string"]
         if output not in validOutputTypes:
             raise ValueError("output must be one of %r." % validOutputTypes)
         args = {}
@@ -292,6 +312,11 @@ class JsonBridgeHistorian (object):
         if output == "raw":
             return response
         elif output == "json":
+            if len(force_list(tags)) == 1:
+                return items[tags]
+            else:
+                return items
+        elif output == "string":
             return json.dumps(items)
         elif output == "df":
             return jsonToDf(items)
@@ -322,7 +347,7 @@ class JsonBridgeHistorian (object):
             RemoteServerException: When the provided historian could not be found, or the API response contains an error
             RunTimeError/HTTPError: No response from the API
         """
-        validOutputTypes = ["raw", "json", "df", "csv"]
+        validOutputTypes = ["raw", "json", "df", "csv", "string"]
         if output not in validOutputTypes:
             raise ValueError("output must be one of %r." % validOutputTypes)
         args = {}
@@ -345,6 +370,11 @@ class JsonBridgeHistorian (object):
         if output == "raw":
             return response
         elif output == "json":
+            if len(force_list(tags)) == 1:
+                return response[tags]
+            else:
+                return response
+        elif output == "string":
             return json.dumps(response)
         elif output == "df":
             return aggToDf(response)
@@ -377,7 +407,7 @@ class JsonBridgeHistorian (object):
             RemoteServerException: When the provided historian could not be found, or the API response contains an error
             RunTimeError/HTTPError: No response from the API
         """
-        validOutputTypes = ["raw", "json", "df", "csv"]
+        validOutputTypes = ["raw", "json", "df", "csv", "string"]
         if output not in validOutputTypes:
             raise ValueError("output must be one of %r." % validOutputTypes)
 
@@ -415,6 +445,11 @@ class JsonBridgeHistorian (object):
         if output == "raw":
             return response
         elif output == "json":
+            if len(force_list(tags)) == 1:
+                return response[tags]
+            else:
+                return response
+        elif output == "string":
             return json.dumps(response)
         elif output == "df":
             return aggToDf(response)
@@ -442,7 +477,7 @@ class JsonBridgeHistorian (object):
             RemoteServerException: When the provided historian could not be found, or the API response contains an error
             RunTimeError/HTTPError: No response from the API
         """
-        validOutputTypes = ["raw", "json", "df", "csv"]
+        validOutputTypes = ["raw", "json", "df", "csv", "string"]
         if output not in validOutputTypes:
             raise ValueError("output must be one of %r." % validOutputTypes)
         ret = {}
@@ -452,9 +487,9 @@ class JsonBridgeHistorian (object):
 
         response = self._matchArgumentCardinality(tags, ret)
 
-        if output == "raw":
+        if output == "raw" or output == "json":
             return response
-        elif output == "json":
+        elif output == "string":
             return json.dumps(response)
         elif output == "df":
             df = jsonToDf({"No. of Points": response})
@@ -462,7 +497,7 @@ class JsonBridgeHistorian (object):
         return None
 
 
-    def _writeDataPoints(self, data:dict) -> bool:
+    def _writeDataPoints(self, data) -> bool:
         rawdata = {}
         for tag in list(data.keys()):
             rawdps = []
@@ -519,8 +554,29 @@ class JsonBridgeHistorian (object):
             return ret[proto]
         except KeyError:
             raise KeyError("Could not find tag(s): " + str(proto))
+        
+    def writePoints(self, tag:str, points:Union[dict,list]) -> bool:
+        """
+        Writes point data to a single existing tag.
 
+        Args:
+            tag: Id of tag to write to
+            points: The points to add to the tag (takes form [{value: ... , timestamp: ... , status: ...}, { ... }, ... ])
 
+        Returns:
+            Success: A boolean representing the successful push of data.
+        """
+        points = force_list(points)
+
+        datapoints = {tag : []}
+        for point in points:
+            if type(point["timestamp"]) is datetime:
+                point["timestamp"] = point["timestamp"].timestamp()
+            datapoints[tag].append(DataPoint(point["value"],point["timestamp"],point["status"]))
+
+        return self._writeDataPoints(datapoints)
+    
+    
     def batchCSVImport(self, filepaths: Union[str,list]) -> bool:
         """
         Writes tag data from one or more csv file to Ingenuity. Will create a tag if it does not exist.
@@ -586,7 +642,7 @@ class JsonBridgeHistorian (object):
         return all(success)
 
 
-class DataPoint(object):
+class DataPoint(object): # LEGACY OBJECT
     """This class represents a data point which has a value and a python timestamp, and optionally a status of 'OK' or 'BAD'.
        Timestamp is stored in epochseconds. The constructor will convert datetime and tuples into floating point epoch seconds.
     """
@@ -724,3 +780,6 @@ def get_default_historian_name(eigenserver=None) -> str:
         eigenserver = get_default_server()
 
     return eigenserver.getDefaultDataSource("historian")
+
+def createPoint(value:Union[str,int,float],timestamp:int,status:str):
+    return {"value": value, "timestamp": timestamp, "status": status}
